--!parent

-- ************************************************************************
-- Script: BYTEARRAY FOR D10
-- Version:	0.6
-- Date: 10.10.2021
--
-- Requirements:
-- + compress/uncompress require zlib xtra
-- ************************************************************************

-- public
property length
property position
property endian
property ilk -- for overwriting x.ilk -> #byteArray

-- private
property _data
property _pow

----------------------------------------
-- on ByteArray(me, [intSize], [intInitialValue])
-- on ByteArray(me, [str])
----------------------------------------
on new (me, p1, p2)
	me.ilk = #ByteArray
	me.endian = #LittleEndian
	if stringP(p1) then
		me._data = p1
		me.length = p1.length
	else
		me._data = ""
		if integerP(p1) then
			if not integerP(p2) then p2=0
			repeat with i = 1 to p1
				put numtochar(p2) after _data
			end repeat
			me.length = p1
		else
			me.length = 0
		end if
	end if
	me.position = 1
	me._pow = [1, 256, 65536, 16777216]
	return me
end

----------------------------------------
-- for overwriting x.ilk() or ilk(x) -> #byteArray
----------------------------------------
on ilk (me)
	return #ByteArray
end

----------------------------------------
-- "<ByteArrayObject length = 0 ByteArray = <Void>> "
-- "<ByteArrayObject length = 12 ByteArray = 0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x57, 0x6f, 0x72, 0x6c > "
----------------------------------------
on string (me)
	str = "<ByteArrayObject length = "&me.length&" ByteArray ="
	if me.length=0 then
		put " <Void>" after str
	else
		cnt = min(me.length, 10)
		repeat with i = 1 to cnt
			put " 0x" & rgb(chartonum(me._data.char[i]),0,0).hexString().char[2..3] & "," after str
		end repeat
		delete the last char of str
	end if
	put "> " after str
	return str
end

--**************************************
-- BRACKET ACCESS
--**************************************

----------------------------------------
-- put aByteArray[23]
----------------------------------------
on getAt (me, pos)
	return chartonum(me._data.char[pos])
end

----------------------------------------
-- aByteArray[23] = 123
----------------------------------------
on setAt (me, pos, val)
	put numtochar(val) into char pos of _data
end


--**************************************
-- PUBLIC METHODS
--**************************************

----------------------------------------
-- only implemented as method (otherwise it needs to be updated after
-- every action)
----------------------------------------
on bytesRemaining (me)
	return me.length - me.position + 1
end

----------------------------------------
--
----------------------------------------
on length (me)
	return me.length
end

----------------------------------------
--
----------------------------------------
on readBoolean (me)
	ret = chartonum(me._data[me.position])<>0
	me.position = me.position + 1
	return ret
end

----------------------------------------
--
----------------------------------------
on writeBoolean (me, boolVal)
	put numtochar(boolVal<>0) into char position of _data
	me.position = me.position+1
	me.length = max(me.length, me._data.length)
end

----------------------------------------
--
----------------------------------------
on readInt8 (me)
	v = chartonum(me._data.char[me.position])
	me.position = me.position+1
	return v
end

----------------------------------------
--
----------------------------------------
on readInt16 (me)
	if endian = #littleEndian then
		v = chartonum(me._data.char[me.position+0]) + 256*chartonum(me._data.char[me.position+1])
	else
		v = chartonum(me._data.char[me.position+1]) + 256*chartonum(me._data.char[me.position+0])
	end if
	me.position = me.position + 2
	return v
end

----------------------------------------
--
----------------------------------------
on readInt32 (me)
	if endian = #littleEndian then
		v = chartonum(me._data.char[me.position+0]) + 256*chartonum(me._data.char[me.position+1]) + 65536*chartonum(me._data.char[me.position+2]) + 16777216*chartonum(me._data.char[me.position+3])
	else
		v = chartonum(me._data.char[me.position+3]) + 256*chartonum(me._data.char[me.position+2]) + 65536*chartonum(me._data.char[me.position+1]) + 16777216*chartonum(me._data.char[me.position+0])
	end if
	me.position = me.position + 4
	return v
end

----------------------------------------
--
----------------------------------------
on writeInt8 (me, intVal)
	put numtochar(intVal) into char me.position of _data
	me.position = me.position + 1
	me.length = max(length, me._data.length)
end

----------------------------------------
--
----------------------------------------
on writeInt16 (me, intVal)
	if me.endian=#LittleEndian then
		repeat with i = 1 to 2
			div = me._pow[i] -- integer(power(256,i))
			if div>intVal then
				put numtochar(0) into char (me.position+i-1) of _data
			else
				k = (intVal/div) mod 256
				put numtochar(k) into char (me.position+i-1) of _data
			end if
		end repeat
	else
		repeat with i = 1 to 2
			div = me._pow[3-i]
			if div>intVal then
				put numtochar(0) into char (me.position+i-1) of _data
			else
				k = (intVal/div) mod 256
				put numtochar(k) into char (me.position+i-1) of _data
			end if
		end repeat
	end if
	me.position = me.position + 2
	me.length = max(me.length, me._data.length)
end

----------------------------------------
--
----------------------------------------
on writeInt32 (me, intVal)
	if endian = #littleEndian then
		repeat with i = 1 to 4
			div = me._pow[i] -- integer(power(256,i))
			if div>intVal then
				put numtochar(0) into char (me.position+i-1) of _data
			else
				k = (intVal/div) mod 256
				put numtochar(k) into char (me.position+i-1) of _data
			end if
		end repeat
	else
		repeat with i = 1 to 4
			div = me._pow[5-i]
			if div > intVal then
				put numtochar(0) into char (me.position+i-1) of _data
			else
				k = (intVal/div) mod 256
				put numtochar(k) into char (me.position+i-1) of _data
			end if
		end repeat
	end if
	me.position =me. position + 4
	me.length = max(me.length, me._data.length)
end

----------------------------------------
-- Reads an IEEE 754 single-precision (32-bit) floating-point number from the byte stream
-- http://de.wikipedia.org/wiki/IEEE_754
----------------------------------------
on readFloat32 (me)
	if endian = #littleEndian then
		buf = ""
		repeat with i = 4 down to 1
			put me._data.char[me.position+i-1] after buf
		end repeat
	else
		buf = me._data.char[me.position..me.position+3]
	end if

	c1 = charToNum(buf.char[1])
	c2 = charToNum(buf.char[2])

	neg = bitAnd(c1, 128)
	exp = bitAnd(c1, 127) * 2 + bitAnd(c2, 128) / 128 - 127 --1023

	if exp = 128 then
		return void
	else if exp = -127 then
		exp = -126
		mnt = 0
	else mnt = 1

	m = 0.5
	i = 64
	repeat with c = 1 to 3
		cn = charToNum(buf.char[c+1])
		repeat while i
			if bitAnd(cn, i) then mnt = mnt + m
			m = m / 2
			i = i / 2
		end repeat
		i = 128
	end repeat

	me.position = me.position + 4
	if neg then return -mnt * power(2, exp)
	return mnt * power(2, exp)
end

----------------------------------------
-- Reads an IEEE 754 double-precision (64-bit) floating-point number from the byte stream
----------------------------------------
on readFloat64 (me)
	if endian = #littleEndian then
		buf = ""
		repeat with i = 8 down to 1
			put me._data.char[me.position+i-1] after buf
		end repeat
	else
		buf = me._data.char[me.position..me.position+7]
	end if

	c1 = charToNum(buf.char[1])
	c2 = charToNum(buf.char[2])

	neg = bitAnd(c1, 128)
	exp = bitAnd(c1, 127) * 16 + bitAnd(c2, 240) / 16 - 1023

	if exp = 1024 then
		return void
	else if exp = -1023 then
		exp = -1022
		mnt = 0
	else mnt = 1
	m = 0.5
	i = 8
	repeat with c = 1 to 7
		cn = charToNum(buf.char[c+1])
		repeat while i
			if bitAnd(cn, i) then mnt = mnt + m
			m = m / 2
			i = i / 2
		end repeat
		i = 128
	end repeat

	me.position = me.position + 8
	if neg then return -mnt * power(2, exp)
	return mnt * power(2, exp)
end

----------------------------------------
-- Writes an IEEE 754 single-precision (32-bit) floating-point number to the byte stream.
----------------------------------------
on writeFloat32 (me, floatVal)
	floatVal = float(floatVal)
	if floatVal = 0 then
		repeat with i = 1 to 4
			put numToChar(0) into char (me.position+i-1) of _data
		end repeat
		me.position = me.position + 4
		me.length = max(me.length, me._data.length)
		return
	end if
	if floatVal < 0 then
		floatVal = -floatVal
		neg = 128
	else neg = 0
	exp = 127
	if floatVal >= 1 then
		repeat while floatVal >= 2
			floatVal = floatVal / 2
			exp = exp + 1
		end repeat
	else
		repeat while floatVal < 1
			floatVal = floatVal * 2
			exp = exp - 1
		end repeat
	end if
	floatVal = floatVal - 1
	m = 0.5
	i = 64
	rslt = numToChar(bitAnd(exp, 254) / 2 + neg)
	val = bitAnd(exp, 1) * 128
	repeat with c = 1 to 3
		repeat while i
			if floatVal >= m then
				floatVal = floatVal - m
				val = val + i
			end if
			m = m / 2.0
			i = i / 2
		end repeat
		put numToChar(val) after rslt
		i = 128
		val = 0
	end repeat
	if endian = #littleEndian then
		repeat with i = 1 to 4
			put rslt.char[5-i] into char (me.position+i-1) of _data
		end repeat
	else
		put rslt into char (me.position) to (me.position+3) of _data
	end if
	me.position = me.position + 4
	me.length = max(me.length, me._data.length)
end

----------------------------------------
-- Writes an IEEE 754 double-precision (64-bit) floating-point number to the byte stream.
----------------------------------------
on writeFloat64 (me, floatVal)
	floatVal = float(floatVal)
	if floatVal = 0 then
		repeat with i = 1 to 8
			put numToChar(0) into char (me.position+i-1) of _data
		end repeat
		me.position = me.position + 8
		me.length = max(me.length, me._data.length)
		return
	end if
	if floatVal < 0 then
		floatVal = -floatVal
		neg = 128
	else neg = 0
	exp = 1023
	if floatVal >= 1 then
		repeat while floatVal >= 2
			floatVal = floatVal / 2
			exp = exp + 1
		end repeat
	else
		repeat while floatVal < 1
			floatVal = floatVal * 2
			exp = exp - 1
		end repeat
	end if
	floatVal = floatVal - 1
	m = 0.5
	i = 8
	rslt = numToChar(bitAnd(exp, 2032) / 16 + neg)
	val = bitAnd(exp, 15) * 16
	repeat with c = 1 to 7
		repeat while i
			if floatVal >= m then
				floatVal = floatVal - m
				val = val + i
			end if
			m = m / 2
			i = i / 2
		end repeat
		put numToChar(val) after rslt
		i = 128
		val = 0
	end repeat
	if endian = #littleEndian then
		repeat with i = 1 to 8
			put rslt.char[9-i] into char (me.position+i-1) of _data
		end repeat
	else
		put rslt into char (me.position) to (me.position+7) of _data
	end if
	me.position = me.position + 8
	me.length = max(me.length, me._data.length)
end

----------------------------------------
-- Reads part of a ByteArray into another byte array. If an existing ByteArray is passed as input
-- read data shall be appended to that byte array starting intOffset position.
-- [byteArray], [intOffset]
----------------------------------------
on readByteArray (me, intLen, byteArray, intOffset)
	if byteArray.ilk = #byteArray then
		byteArray.writeByteArray(me, intOffset, intLen)
	else
		return me.script.new(me._data.char[me.position..me.position+intLen-1])
	end if
end

----------------------------------------
-- [intOffset],[ intLen]
----------------------------------------
on writeByteArray (me, byteArray, intOffset, intLen)
	if voidP(intOffset) then intOffset=0
	if voidP(intLen) then intLen = byteArray.length-intOffset
	put byteArray.me._data.char[intOffset+1..intOffset+intLen] into me._data.char[me.position..me.position+intLen-1]
	me.position = me.position + intLen
	me.length = max(me.length, me._data.length)
end

----------------------------------------
 --[strCharSet]
----------------------------------------
on readString (me)
	len = me.readInt32()
	return me._data.char[me.position..me.position + len - 1]
end

----------------------------------------
-- [strCharSet]
----------------------------------------
on writeString (me, strValue)
	len = strValue.length
	me.writeInt32(len)
	put strValue into me._data.char[me.position..me.position + len - 1]
end

----------------------------------------
-- [strCharSet]
----------------------------------------
on readRawString (me, intLen)
	str = me._data.char[me.position..me.position + intLen - 1]
	me.position = me.position + intLen
	return str
end

----------------------------------------
-- [strCharSet]
----------------------------------------
on writeRawString (me, strValue, intLen)
	len = min(intLen, strValue.length)
	put strValue.char[1..len] into char (me.position) to (me.position + len - 1) of _data
	me.position = me.position + len
	me.length = max(me.length, me._data.length)
end

----------------------------------------
--
----------------------------------------
on compress (me)
	me._data = zx_gzcompress_string(me._data)
	me.length = me._data.length
	me.position = 1
end

----------------------------------------
--
----------------------------------------
on uncompress (me)
	me._data = zx_gzuncompress_string(me._data)
	me.length = me._data.length
	me.position = 1
end
