--!parent

--****************************************************************************
-- @file LSW CORE LIB: FILE
-- @author Valentin Schmidt
-- @version 0.2
--****************************************************************************

property $
property ancestor

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	if voidP(value("bytearray")) then
		-- Director 10.x
		$.import("bytearray_d10")
		$.import("filesystem")
		$.import("zlib")
		me.ancestor = $.include(me.LIBDIR & "file_d10.ls").new()
	else
		-- Director 11.5+
		me.ancestor = $.include(me.LIBDIR & "file_d11.ls").new()
	end if
	return me
end

----------------------------------------
-- Returns file type
-- @param {string} tFilename
-- @return {string}
----------------------------------------
on getFileType (me, tFilename)
	tFilename = me.getFileName(tFilename)
	od = the itemdelimiter
	the itemdelimiter = "."
	if tFilename.item.count=1 then
		the itemdelimiter = od
		return ""
	end if
	tType = the last item of tFilename
	the itemdelimiter = od
	-- to lower
	alphabet = "abcdefghijklmnopqrstuvwxyz"
	cnt = tType.length
	repeat with i = 1 to cnt
		pos = offset(tType.char[i],alphabet)
		if pos > 0 then put alphabet.char[pos] into char i of tType
	end repeat
	return tType
end

----------------------------------------
-- Extracts filename from path
-- @param {string} tPath
-- @return {string}
----------------------------------------
on getFileName (me, tPath)
	if tPath contains "/" then pd = "/"
	else if tPath contains "\" then pd = "\"
	else pd = ":"
	od = the itemdelimiter
	the itemdelimiter = pd
	tFileName = the last item of tPath
	the itemdelimiter = od
	return tFileName
end

----------------------------------------
-- Extracts path to directory from path
-- @param {string} tPath
-- @return {string}
----------------------------------------
on getFilePath (me, tPath)
	if tPath contains "/" then pd = "/"
	else if tPath contains "\" then pd = "\"
	else pd = ":"
	od = the itemdelimiter
	the itemdelimiter = pd
	delete the last item of tPath
	the itemdelimiter = od
	if the last char of tPath=":" then put "\" after tPath
	return tPath
end

----------------------------------------
-- Extracts basename from path
-- @param {string} tPath
-- @return {string}
----------------------------------------
on getBaseName (me, tPath)
	fn = me.getFileName(tPath)
	cnt = fn.length
	repeat with i = cnt down to 1
		if fn.char[i]="." then return fn.char[1..i-1]
	end repeat
	return fn
end


----------------------------------------
-- Displays open file dialog
-- @param {string} tTitle
-- @param {string} filterMask
-- @param {string} defaultFileName
-- @return {string}
----------------------------------------
on selectFileOpen (me, tTitle, filterMask, defaultFileName)
	fio = xtra("fileIO").new()
	fio.setFilterMask(filterMask)
	fn = fio.displayOpen()
	return string(fn)
end

----------------------------------------
-- Displays save file dialog
-- @param {string} tTitle
-- @param {string} filterMask
-- @param {string} defaultFileName
-- @return {string}
----------------------------------------
on selectFileSave (me, tTitle, filterMask, defaultFileName)
	fio = xtra("fileIO").new()
	fio.setFilterMask(filterMask)
	fn = fio.displaySave(tTitle, defaultFileName)
	return string(fn)
end

----------------------------------------
-- Select folder via save file dialog
-- @param {string} tTitle
-- @return {string}
----------------------------------------
on selectFolder (me, tTitle)
	if voidP(tTitle) then tTitle = "Select Folder"
	fio = new(xtra "fileIO")
	fn = fio.displaySave(tTitle, "*.*")
	err = fio.status()
	if string(fn)=EMPTY then return ""
	if err<>0 then
		put "Save File Error:" && fio.error(err)
		--exit
		return ""
	end if
	pd = the last char of _movie.path
	od = _player.itemDelimiter
	_player.itemDelimiter = pd
	delete the last item of fn
	_player.itemDelimiter = od
	put pd after fn
	return fn
end
